"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const TestRenderer = require("react-test-renderer");
const lodashGet = require("lodash.get");
const _1 = require(".");
const simpleMachineConfig = {
    id: "light",
    initial: "green",
    states: {
        green: {
            on: { TIMER: "yellow" },
        },
        yellow: {
            on: { TIMER: "red" },
        },
        red: {
            on: { TIMER: "green" },
        },
    },
};
const simpleMachineConfigWithSideEffect = {
    id: "switch",
    initial: "off",
    states: {
        off: {
            on: {
                '': {
                    target: "on",
                    actions: ["switched"],
                },
            },
        },
        on: {},
    },
};
describe("testing useMachine", () => {
    it("should successfully render a component with a simple machine and call useEffect", () => {
        const spy = jest.spyOn(React, "useEffect");
        const SimpleTestComponent = () => {
            const machine = _1.useMachine(simpleMachineConfig, {}, {});
            return (React.createElement("div", null, machine.state.value));
        };
        let testComponent = TestRenderer.create(React.createElement(SimpleTestComponent, null));
        const renderedJSON = testComponent.toJSON();
        expect(lodashGet(renderedJSON, 'children[0]'))
            .toBe("green");
        testComponent && testComponent.unmount();
        expect(spy).toHaveBeenCalled();
        spy.mockRestore();
    });
    it("should successfully re-render an updated component when send is called to change machine state", () => {
        const TestComponentWithAction = () => {
            const machine = _1.useMachine(simpleMachineConfig, {}, {});
            const sendTimer = () => {
                machine.send("TIMER");
            };
            return (React.createElement(React.Fragment, null,
                React.createElement("div", null, machine.state.value),
                React.createElement("button", { onClick: sendTimer })));
        };
        const testComponent = TestRenderer.create(React.createElement(TestComponentWithAction, null));
        // check starting component state
        const initialRenderedJSON = testComponent.toJSON();
        expect(lodashGet(initialRenderedJSON, '[0].children[0]'))
            .toBe("green");
        // click button
        TestRenderer.act(() => testComponent.root && testComponent.root.findByType("button").props.onClick());
        const afterClickRenderedJSON = testComponent.toJSON();
        // check component state after click
        expect(lodashGet(afterClickRenderedJSON, '[0].children[0]'))
            .toBe("yellow");
    });
    it("should successfully execute a side-effect when one is included in the machine config", () => {
        const actions = {
            switched: () => null
        };
        const spy = jest.spyOn(actions, "switched");
        const TestComponentWithSideEffect = () => {
            const machine = _1.useMachine(simpleMachineConfigWithSideEffect, {
                actions: {
                    switched: actions.switched,
                },
            }, {});
            return null;
        };
        TestRenderer.create(React.createElement(TestComponentWithSideEffect, null));
        expect(spy).toHaveBeenCalled();
        spy.mockRestore();
    });
});
